#!/usr/bin/env python3
"""
Simple integrated UI that properly uses the Protocol.
This replaces the complex integration attempt with a cleaner approach.
"""

import sys
import os
from PyQt5.QtWidgets import QApplication, QMainWindow, QWidget, QVBoxLayout, QPushButton, QLabel
from PyQt5.QtCore import Qt, QThread, pyqtSignal, QTimer

# Add Protocol directory to path
sys.path.insert(0, os.path.join(os.path.dirname(os.path.dirname(__file__)), 'Protocol'))

from protocol import IcingProtocol


class ProtocolWorker(QThread):
    """Worker thread for protocol operations."""
    
    message_received = pyqtSignal(str)
    state_changed = pyqtSignal(str)
    
    def __init__(self, protocol):
        super().__init__()
        self.protocol = protocol
        self.running = True
        self.processed_count = 0
        
    def run(self):
        """Monitor protocol for new messages."""
        while self.running:
            try:
                # Check for new messages
                if hasattr(self.protocol, 'inbound_messages'):
                    new_messages = self.protocol.inbound_messages[self.processed_count:]
                    for msg in new_messages:
                        self.processed_count += 1
                        msg_type = msg.get('type', 'UNKNOWN')
                        self.message_received.emit(f"Received: {msg_type}")
                        
                        # Handle specific message types
                        if msg_type == 'PING_REQUEST' and self.protocol.auto_responder:
                            self.state_changed.emit("Responding to PING...")
                        elif msg_type == 'PING_RESPONSE':
                            self.state_changed.emit("PING response received")
                        elif msg_type == 'HANDSHAKE':
                            self.state_changed.emit("Handshake message received")
                        elif msg_type == 'ENCRYPTED_MESSAGE':
                            self.state_changed.emit("Encrypted message received")
                            
                # Check protocol state
                if self.protocol.state.get('key_exchange_complete'):
                    self.state_changed.emit("Key exchange complete!")
                    
                self.msleep(100)
            except Exception as e:
                print(f"Worker error: {e}")
                self.msleep(100)
                
    def stop(self):
        self.running = False
        self.quit()
        self.wait()


class SimpleProtocolUI(QMainWindow):
    """Simple UI to demonstrate protocol integration."""
    
    def __init__(self):
        super().__init__()
        self.setWindowTitle("Simple Protocol Integration")
        self.setGeometry(100, 100, 400, 500)
        
        # Create protocol instances
        self.protocol1 = IcingProtocol()
        self.protocol2 = IcingProtocol()
        
        # Exchange identity keys
        self.protocol1.set_peer_identity(self.protocol2.identity_pubkey.hex())
        self.protocol2.set_peer_identity(self.protocol1.identity_pubkey.hex())
        
        # Enable auto-responder on protocol 2
        self.protocol2.auto_responder = True
        
        # Create UI
        central_widget = QWidget()
        self.setCentralWidget(central_widget)
        layout = QVBoxLayout()
        central_widget.setLayout(layout)
        
        # Info labels
        self.info_label = QLabel("Protocol Status")
        self.info_label.setAlignment(Qt.AlignCenter)
        layout.addWidget(self.info_label)
        
        self.port_label = QLabel(f"Protocol 1: port {self.protocol1.local_port}\n"
                                f"Protocol 2: port {self.protocol2.local_port}")
        layout.addWidget(self.port_label)
        
        # Status display
        self.status_label = QLabel("Ready")
        self.status_label.setStyleSheet("QLabel { background-color: #f0f0f0; padding: 10px; }")
        layout.addWidget(self.status_label)
        
        # Message log
        self.log_label = QLabel("Message Log:\n")
        self.log_label.setAlignment(Qt.AlignTop)
        self.log_label.setStyleSheet("QLabel { background-color: #ffffff; padding: 10px; }")
        self.log_label.setMinimumHeight(200)
        layout.addWidget(self.log_label)
        
        # Buttons
        self.connect_btn = QPushButton("1. Connect")
        self.connect_btn.clicked.connect(self.do_connect)
        layout.addWidget(self.connect_btn)
        
        self.ping_btn = QPushButton("2. Send PING")
        self.ping_btn.clicked.connect(self.do_ping)
        self.ping_btn.setEnabled(False)
        layout.addWidget(self.ping_btn)
        
        self.handshake_btn = QPushButton("3. Send Handshake")
        self.handshake_btn.clicked.connect(self.do_handshake)
        self.handshake_btn.setEnabled(False)
        layout.addWidget(self.handshake_btn)
        
        self.derive_btn = QPushButton("4. Derive Keys")
        self.derive_btn.clicked.connect(self.do_derive)
        self.derive_btn.setEnabled(False)
        layout.addWidget(self.derive_btn)
        
        self.encrypt_btn = QPushButton("5. Send Encrypted Message")
        self.encrypt_btn.clicked.connect(self.do_encrypt)
        self.encrypt_btn.setEnabled(False)
        layout.addWidget(self.encrypt_btn)
        
        # Create workers
        self.worker1 = ProtocolWorker(self.protocol1)
        self.worker1.message_received.connect(lambda msg: self.log_message(f"P1: {msg}"))
        self.worker1.state_changed.connect(lambda state: self.update_status(f"P1: {state}"))
        self.worker1.start()
        
        self.worker2 = ProtocolWorker(self.protocol2)
        self.worker2.message_received.connect(lambda msg: self.log_message(f"P2: {msg}"))
        self.worker2.state_changed.connect(lambda state: self.update_status(f"P2: {state}"))
        self.worker2.start()
        
        # Wait timer for protocol startup
        QTimer.singleShot(1000, self.on_ready)
        
    def on_ready(self):
        """Called when protocols are ready."""
        self.status_label.setText("Protocols ready. Click Connect to start.")
        
    def log_message(self, msg):
        """Add message to log."""
        current = self.log_label.text()
        self.log_label.setText(current + msg + "\n")
        
    def update_status(self, status):
        """Update status display."""
        self.status_label.setText(status)
        
    def do_connect(self):
        """Connect protocol 1 to protocol 2."""
        try:
            self.protocol1.connect_to_peer(self.protocol2.local_port)
            self.log_message("Connected to peer")
            self.connect_btn.setEnabled(False)
            self.ping_btn.setEnabled(True)
            
            # Generate ephemeral keys
            self.protocol1.generate_ephemeral_keys()
            self.log_message("Generated ephemeral keys")
        except Exception as e:
            self.log_message(f"Connection error: {e}")
            
    def do_ping(self):
        """Send PING request."""
        try:
            self.protocol1.send_ping_request(cipher_type=1)  # ChaCha20
            self.log_message("Sent PING request")
            self.ping_btn.setEnabled(False)
            self.handshake_btn.setEnabled(True)
        except Exception as e:
            self.log_message(f"PING error: {e}")
            
    def do_handshake(self):
        """Send handshake."""
        try:
            self.protocol1.send_handshake()
            self.log_message("Sent handshake")
            self.handshake_btn.setEnabled(False)
            
            # Enable derive after a delay (to allow response)
            QTimer.singleShot(500, lambda: self.derive_btn.setEnabled(True))
        except Exception as e:
            self.log_message(f"Handshake error: {e}")
            
    def do_derive(self):
        """Derive keys."""
        try:
            self.protocol1.derive_hkdf()
            self.log_message("Derived keys")
            self.derive_btn.setEnabled(False)
            self.encrypt_btn.setEnabled(True)
            
            # Check if protocol 2 also completed
            if self.protocol2.state.get('key_exchange_complete'):
                self.log_message("Both protocols have completed key exchange!")
        except Exception as e:
            self.log_message(f"Derive error: {e}")
            
    def do_encrypt(self):
        """Send encrypted message."""
        try:
            test_msg = "Hello, encrypted world!"
            self.protocol1.send_encrypted_message(test_msg)
            self.log_message(f"Sent encrypted: '{test_msg}'")
            
            # Check if protocol 2 can decrypt
            QTimer.singleShot(100, self.check_decryption)
        except Exception as e:
            self.log_message(f"Encryption error: {e}")
            
    def check_decryption(self):
        """Check if protocol 2 received and can decrypt."""
        for i, msg in enumerate(self.protocol2.inbound_messages):
            if msg.get('type') == 'ENCRYPTED_MESSAGE':
                try:
                    decrypted = self.protocol2.decrypt_received_message(i)
                    self.log_message(f"P2 decrypted: '{decrypted}'")
                    self.log_message("SUCCESS! Full protocol flow complete.")
                except Exception as e:
                    self.log_message(f"Decryption error: {e}")
                break
                
    def closeEvent(self, event):
        """Clean up on close."""
        self.worker1.stop()
        self.worker2.stop()
        
        if self.protocol1.server_listener:
            self.protocol1.server_listener.stop()
        if self.protocol2.server_listener:
            self.protocol2.server_listener.stop()
            
        for conn in self.protocol1.connections:
            conn.close()
        for conn in self.protocol2.connections:
            conn.close()
            
        event.accept()


def main():
    app = QApplication(sys.argv)
    window = SimpleProtocolUI()
    window.show()
    sys.exit(app.exec_())


if __name__ == '__main__':
    main()