#!/usr/bin/env python3
"""
Example of proper Protocol integration with handshake flow.
"""

import sys
import os
import time
import threading

# Add directories to path
sys.path.insert(0, os.path.join(os.path.dirname(__file__), 'Protocol'))
sys.path.insert(0, os.path.join(os.path.dirname(__file__), 'UI'))

from protocol import IcingProtocol
from protocol_phone_client import ProtocolPhoneClient

def demo_handshake():
    """Demonstrate the proper handshake flow between two protocols."""
    print("\n=== Protocol Handshake Demo ===\n")
    
    # Create two protocol instances
    protocol1 = IcingProtocol()
    protocol2 = IcingProtocol()
    
    print(f"Protocol 1 listening on port: {protocol1.local_port}")
    print(f"Protocol 1 identity: {protocol1.identity_pubkey.hex()[:32]}...")
    print(f"Protocol 2 listening on port: {protocol2.local_port}")
    print(f"Protocol 2 identity: {protocol2.identity_pubkey.hex()[:32]}...")
    print()
    
    # Wait for listeners to start
    time.sleep(1)
    
    # Exchange identity keys - these are already valid EC public keys
    try:
        protocol1.set_peer_identity(protocol2.identity_pubkey.hex())
        protocol2.set_peer_identity(protocol1.identity_pubkey.hex())
        print("Identity keys exchanged successfully")
    except Exception as e:
        print(f"Error exchanging identity keys: {e}")
        return
    
    # Enable auto-responder on protocol2
    protocol2.auto_responder = True
    
    print("\n1. Protocol 1 connects to Protocol 2...")
    try:
        protocol1.connect_to_peer(protocol2.local_port)
        time.sleep(0.5)
    except Exception as e:
        print(f"Connection failed: {e}")
        return
    
    print("\n2. Protocol 1 generates ephemeral keys...")
    protocol1.generate_ephemeral_keys()
    
    print("\n3. Protocol 1 sends PING request (requesting ChaCha20)...")
    protocol1.send_ping_request(cipher_type=1)
    time.sleep(0.5)
    
    print("\n4. Protocol 2 auto-responds with PING response...")
    # Auto-responder handles this automatically
    time.sleep(0.5)
    
    print("\n5. Protocol 1 sends handshake...")
    protocol1.send_handshake()
    time.sleep(0.5)
    
    print("\n6. Protocol 2 auto-responds with handshake...")
    # Auto-responder handles this automatically
    time.sleep(0.5)
    
    print("\n7. Both derive keys...")
    protocol1.derive_hkdf()
    # Protocol 2 auto-derives in auto-responder mode
    time.sleep(0.5)
    
    print("\n=== Handshake Complete ===")
    print(f"Protocol 1 - Key exchange complete: {protocol1.state['key_exchange_complete']}")
    print(f"Protocol 2 - Key exchange complete: {protocol2.state['key_exchange_complete']}")
    
    if protocol1.hkdf_key and protocol2.hkdf_key:
        print(f"\nDerived keys match: {protocol1.hkdf_key == protocol2.hkdf_key}")
        print(f"Cipher type: {'ChaCha20-Poly1305' if protocol1.cipher_type == 1 else 'AES-256-GCM'}")
        
        # Test encrypted messaging
        print("\n8. Testing encrypted message...")
        test_msg = "Hello, encrypted world!"
        protocol1.send_encrypted_message(test_msg)
        time.sleep(0.5)
        
        # Check if protocol2 received it
        for i, msg in enumerate(protocol2.inbound_messages):
            if msg['type'] == 'ENCRYPTED_MESSAGE':
                decrypted = protocol2.decrypt_received_message(i)
                print(f"Protocol 2 decrypted: {decrypted}")
                break
    
    # Clean up
    protocol1.server_listener.stop()
    protocol2.server_listener.stop()
    
    for conn in protocol1.connections:
        conn.close()
    for conn in protocol2.connections:
        conn.close()

def demo_ui_integration():
    """Demonstrate UI integration with proper handshake."""
    print("\n\n=== UI Integration Demo ===\n")
    
    # This shows how the UI should integrate the protocol
    print("The UI integration flow:")
    print("1. PhoneManager creates ProtocolPhoneClient instances")
    print("2. Identity keys are exchanged via set_peer_identity()")
    print("3. Ports are exchanged via set_peer_port()")
    print("4. When user initiates call:")
    print("   - Initiator calls initiate_call()")
    print("   - This connects to peer and sends PING request")
    print("5. When user answers call:")
    print("   - Responder calls answer_call()")
    print("   - This enables auto-responder and responds to PING")
    print("6. Protocol messages are processed in _process_protocol_messages()")
    print("7. Handshake completes automatically")
    print("8. HANDSHAKE_DONE signal is emitted")
    print("9. Voice session can start with start_voice_session()")
    print("10. Audio is sent via send_audio()")

def main():
    """Run the demos."""
    print("Protocol Integration Example")
    print("=" * 50)
    
    # Run handshake demo
    demo_handshake()
    
    # Explain UI integration
    demo_ui_integration()
    
    print("\n" + "=" * 50)
    print("Demo complete!")

if __name__ == '__main__':
    main()