#!/usr/bin/env python3
"""
Test script to debug auto-test functionality
"""

import sys
import time
import subprocess
from pathlib import Path

# Add parent directory to path
parent_dir = str(Path(__file__).parent.parent)
if parent_dir not in sys.path:
    sys.path.insert(0, parent_dir)

from integrated_protocol import IntegratedDryBoxProtocol

def test_basic_protocol():
    """Test basic protocol functionality"""
    print("=== Testing Basic Protocol ===")
    
    # Create two protocol instances
    phone1 = IntegratedDryBoxProtocol(mode="receiver")
    phone2 = IntegratedDryBoxProtocol(mode="sender")
    
    # Connect to GSM
    print("Connecting to GSM...")
    if not phone1.connect_gsm():
        print("Phone 1 failed to connect to GSM")
        return False
        
    if not phone2.connect_gsm():
        print("Phone 2 failed to connect to GSM")
        return False
        
    print("✓ Both phones connected to GSM")
    
    # Setup connections
    print("\nSetting up protocol connections...")
    port1 = phone1.setup_protocol_connection()
    port2 = phone2.setup_protocol_connection()
    
    print(f"Phone 1 port: {port1}")
    print(f"Phone 2 port: {port2}")
    
    # Connect to each other
    phone1.setup_protocol_connection(peer_port=port2)
    phone2.setup_protocol_connection(peer_port=port1)
    
    print("✓ Connections established")
    
    # Test key exchange
    print("\nTesting key exchange...")
    
    # Phone 1 initiates
    if phone1.initiate_key_exchange(cipher_type=1):  # ChaCha20
        print("✓ Phone 1 initiated key exchange")
    else:
        print("✗ Phone 1 failed to initiate key exchange")
        return False
    
    # Wait for completion
    timeout = 10
    start = time.time()
    while time.time() - start < timeout:
        if phone1.protocol.state.get("key_exchange_complete"):
            print("✓ Key exchange completed!")
            break
        time.sleep(0.5)
    else:
        print("✗ Key exchange timeout")
        return False
    
    # Test message sending
    print("\nTesting encrypted message...")
    test_msg = "Test message from auto-test"
    phone1.send_encrypted_message(test_msg)
    print(f"✓ Sent: {test_msg}")
    
    # Give time for message to be received
    time.sleep(2)
    
    # Clean up
    phone1.close()
    phone2.close()
    
    return True


def test_voice_safe():
    """Test voice functionality safely"""
    print("\n=== Testing Voice (Safe Mode) ===")
    
    # Check if input.wav exists
    input_file = Path(__file__).parent / "input.wav"
    if not input_file.exists():
        print("✗ input.wav not found")
        print("Creating a test audio file...")
        
        # Create a simple test audio file
        try:
            import wave
            import array
            
            with wave.open(str(input_file), 'wb') as wav:
                wav.setnchannels(1)  # Mono
                wav.setsampwidth(2)  # 16-bit
                wav.setframerate(8000)  # 8kHz
                
                # Generate 1 second of 440Hz sine wave
                duration = 1
                samples = []
                for i in range(8000 * duration):
                    t = i / 8000.0
                    sample = int(32767 * 0.5 * (2 * 3.14159 * 440 * t))
                    samples.append(sample)
                
                wav.writeframes(array.array('h', samples).tobytes())
                
            print("✓ Created test audio file")
        except Exception as e:
            print(f"✗ Failed to create audio: {e}")
            return False
    
    print("✓ Audio file ready")
    return True


def main():
    """Run all tests"""
    print("DryBox Auto-Test Functionality Debugger")
    print("=" * 50)
    
    # Start GSM simulator
    print("\nStarting GSM simulator...")
    gsm_path = Path(__file__).parent / "gsm_simulator.py"
    gsm_process = subprocess.Popen(
        [sys.executable, str(gsm_path)],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE
    )
    
    # Wait for GSM to start
    time.sleep(2)
    
    try:
        # Run tests
        if test_basic_protocol():
            print("\n✓ Basic protocol test passed")
        else:
            print("\n✗ Basic protocol test failed")
            
        if test_voice_safe():
            print("\n✓ Voice setup test passed")
        else:
            print("\n✗ Voice setup test failed")
            
    finally:
        # Clean up
        gsm_process.terminate()
        subprocess.run(["pkill", "-f", "gsm_simulator.py"], capture_output=True)
        
    print("\n" + "=" * 50)
    print("Test complete!")


if __name__ == "__main__":
    main()