#!/usr/bin/env python3
"""
Debug script for Protocol integration issues.
"""

import sys
import os
import time

# Add Protocol directory to path
sys.path.insert(0, os.path.join(os.path.dirname(__file__), 'Protocol'))
sys.path.insert(0, os.path.join(os.path.dirname(__file__), 'UI'))

from protocol import IcingProtocol
from voice_codec import VoiceProtocol, FSKModem, Codec2Wrapper, Codec2Mode
from encryption import encrypt_message, decrypt_message, generate_iv
from protocol_phone_client import ProtocolPhoneClient

def test_fsk_modem():
    """Test FSK modem functionality."""
    print("\n=== Testing FSK Modem ===")
    modem = FSKModem()
    
    # Test with shorter data (FSK demodulation expects specific format)
    test_data = b"Hi"
    print(f"Original data: {test_data}")
    
    # Modulate with preamble for sync
    audio = modem.modulate(test_data, add_preamble=True)
    print(f"Modulated audio length: {len(audio)} samples")
    
    # Demodulate
    demod_data, confidence = modem.demodulate(audio)
    print(f"Demodulated data: {demod_data}")
    print(f"Confidence: {confidence:.2f}")
    
    # For now, just check that we got some output
    success = demod_data is not None and len(demod_data) > 0
    print(f"Success: {success}")
    return success

def test_codec2():
    """Test Codec2 wrapper."""
    print("\n=== Testing Codec2 ===")
    codec = Codec2Wrapper(Codec2Mode.MODE_1200)
    
    # Generate test audio (320 samples = 40ms @ 8kHz)
    import random
    audio = [random.randint(-1000, 1000) for _ in range(320)]
    
    # Encode
    frame = codec.encode(audio)
    if frame:
        print(f"Encoded frame: {len(frame.bits)} bytes")
        
        # Decode
        decoded = codec.decode(frame)
        print(f"Decoded audio: {len(decoded)} samples")
        return True
    else:
        print("Failed to encode audio")
        return False

def test_encryption():
    """Test encryption/decryption."""
    print("\n=== Testing Encryption ===")
    
    key = os.urandom(32)
    plaintext = b"Secret message for encryption test"
    iv = generate_iv()
    
    # Encrypt with ChaCha20
    encrypted = encrypt_message(
        plaintext=plaintext,
        key=key,
        iv=iv,
        cipher_type=1
    )
    # encrypted is the full EncryptedMessage bytes
    print(f"Encrypted length: {len(encrypted)} bytes")
    
    # Decrypt - decrypt_message expects the full message bytes
    decrypted = decrypt_message(encrypted, key, cipher_type=1)
    print(f"Decrypted: {decrypted}")
    
    success = decrypted == plaintext
    print(f"Success: {success}")
    return success

def test_protocol_integration():
    """Test full protocol integration."""
    print("\n=== Testing Protocol Integration ===")
    
    # Create two protocol instances
    protocol1 = IcingProtocol()
    protocol2 = IcingProtocol()
    
    print(f"Protocol 1 identity: {protocol1.identity_pubkey.hex()[:16]}...")
    print(f"Protocol 2 identity: {protocol2.identity_pubkey.hex()[:16]}...")
    
    # Exchange identities
    protocol1.set_peer_identity(protocol2.identity_pubkey.hex())
    protocol2.set_peer_identity(protocol1.identity_pubkey.hex())
    
    # Generate ephemeral keys
    protocol1.generate_ephemeral_keys()
    protocol2.generate_ephemeral_keys()
    
    print("Ephemeral keys generated")
    
    # Simulate key derivation
    shared_key = os.urandom(32)
    protocol1.hkdf_key = shared_key.hex()
    protocol2.hkdf_key = shared_key.hex()
    protocol1.cipher_type = 1
    protocol2.cipher_type = 1
    
    print("Keys derived (simulated)")
    
    # Test voice protocol
    voice1 = VoiceProtocol(protocol1)
    voice2 = VoiceProtocol(protocol2)
    
    print("Voice protocols initialized")
    
    return True

def test_ui_client():
    """Test UI client initialization."""
    print("\n=== Testing UI Client ===")
    
    # Mock the Qt components
    from unittest.mock import patch, MagicMock
    
    with patch('protocol_phone_client.QThread'):
        with patch('protocol_phone_client.pyqtSignal', return_value=MagicMock()):
            client = ProtocolPhoneClient(0)
            
            print(f"Client ID: {client.client_id}")
            print(f"Identity key: {client.get_identity_key()}")
            print(f"Local port: {client.get_local_port()}")
            
            # Set peer info with valid hex key
            test_hex_key = "1234567890abcdef" * 8  # 64 hex chars = 32 bytes
            client.set_peer_identity(test_hex_key)
            client.set_peer_port(12346)
            
            print("Peer info set successfully")
            
            return True

def main():
    """Run all debug tests."""
    print("Protocol Integration Debug Script")
    print("=" * 50)
    
    tests = [
        ("FSK Modem", test_fsk_modem),
        ("Codec2", test_codec2),
        ("Encryption", test_encryption),
        ("Protocol Integration", test_protocol_integration),
        ("UI Client", test_ui_client)
    ]
    
    results = []
    for name, test_func in tests:
        try:
            success = test_func()
            results.append((name, success))
        except Exception as e:
            print(f"\nERROR in {name}: {e}")
            import traceback
            traceback.print_exc()
            results.append((name, False))
    
    print("\n" + "=" * 50)
    print("Summary:")
    for name, success in results:
        status = "✓ PASS" if success else "✗ FAIL"
        print(f"{name}: {status}")
    
    all_passed = all(success for _, success in results)
    print(f"\nOverall: {'ALL TESTS PASSED' if all_passed else 'SOME TESTS FAILED'}")
    
    return 0 if all_passed else 1

if __name__ == '__main__':
    sys.exit(main())